'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _graphCrdt = require('graph-crdt');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Asynchronous node interface.
 * @class
 */
var Context = function (_Node) {
  (0, _inherits3.default)(Context, _Node);

  /**
   * Creates a new context.
   * @param {Object} root - The root database instance.
   * @param {Object} [node] - Node configurations.
   */
  function Context(root, node) {
    (0, _classCallCheck3.default)(this, Context);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Context.__proto__ || (0, _getPrototypeOf2.default)(Context)).call(this, node));

    _this.root = root;
    return _this;
  }

  /**
   * Reads a value from the node.
   * @param  {String} field - The field to read.
   * @return {Promise} - Resolves to the value or undefined.
   */


  (0, _createClass3.default)(Context, [{
    key: 'read',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(field) {
        var result;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:

                /** Read from the node. */
                result = this.value(field);

                /** Detect and resolve pointers. */

                if (!(result instanceof Object)) {
                  _context.next = 3;
                  break;
                }

                return _context.abrupt('return', this.root.read(result.edge));

              case 3:
                return _context.abrupt('return', result);

              case 4:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function read(_x) {
        return _ref.apply(this, arguments);
      }

      return read;
    }()

    /**
     * Writes a value to the node.
     * @param  {String} field - Where to write the data.
     * @param  {Mixed} value - A value to write.
     * @return {Promise} - Resolves when the value has written successfully.
     */

  }, {
    key: 'write',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(field, value) {
        var _meta, uid;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:

                /** It's a reference. Save as a pointer. */
                if (value instanceof _graphCrdt.Node) {
                  value = { edge: String(value) };
                }

                _meta = this.meta(), uid = _meta.uid;

                /** Merge the value. */

                _context2.next = 4;
                return this.root.write(uid, (0, _defineProperty3.default)({}, field, value));

              case 4:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function write(_x2, _x3) {
        return _ref2.apply(this, arguments);
      }

      return write;
    }()

    /**
     * Creates a copy of the context.
     * @return {Context} - A new context with the same uid and root.
     */

  }, {
    key: 'new',
    value: function _new() {
      var config = this.meta();

      return new Context(this.root, config);
    }
  }]);
  return Context;
}(_graphCrdt.Node);

exports.default = Context;