'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _graphCrdt = require('graph-crdt');

var _context5 = require('../context');

var _context6 = _interopRequireDefault(_context5);

var _root = require('../root');

var _root2 = _interopRequireDefault(_root);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-env mocha */
describe('A database', function () {
  var db = void 0;

  var settings = _root2.default.configuration;

  beforeEach(function () {
    db = (0, _root2.default)();
  });

  it('should be an object', function () {
    (0, _expect2.default)(db).toBeAn(Object);
  });

  it('should generate a database configuration', function () {
    var config = db[settings];
    (0, _expect2.default)(config).toBeAn(Object);
  });

  it('should create a new graph', function () {
    (0, _expect2.default)(db).toBeA(_graphCrdt.Graph);
  });

  describe('read', function () {

    it('should return null if nothing is found', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      var settings;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return db.read('settings');

            case 2:
              settings = _context.sent;

              (0, _expect2.default)(settings).toBe(null);

            case 4:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    })));
  });

  describe('write', function () {

    it('should save to the graph', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
      var node, name;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return db.write('user', { name: 'Jesse' });

            case 2:
              node = db.value('user');

              (0, _expect2.default)(node).toBeA(_context6.default);

              _context2.next = 6;
              return node.read('name');

            case 6:
              name = _context2.sent;

              (0, _expect2.default)(name).toBe('Jesse');

            case 8:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    })));

    it('should merge updates, not overwrite', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
      var user;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return db.write('user', { name: 'Bob' });

            case 2:
              _context3.next = 4;
              return db.write('user', { handle: 'BobTheGreat' });

            case 4:
              user = db.value('user');
              _context3.next = 7;
              return user.read('name');

            case 7:
              _context3.t0 = _context3.sent;
              (0, _expect2.default)(_context3.t0).toBe('Bob');
              _context3.next = 11;
              return user.read('handle');

            case 11:
              _context3.t1 = _context3.sent;
              (0, _expect2.default)(_context3.t1).toBe('BobTheGreat');

            case 13:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    })));

    it('should return the written context', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
      var user, name;
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return db.write('user', {
                name: 'Bob'
              });

            case 2:
              user = _context4.sent;


              (0, _expect2.default)(user).toBeA(_context6.default);
              (0, _expect2.default)(user).toBe(db.value('user'));

              _context4.next = 7;
              return user.read('name');

            case 7:
              name = _context4.sent;

              (0, _expect2.default)(name).toBe('Bob');

            case 9:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, undefined);
    })));
  });
});