'use strict';

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-env mocha*/
describe('A database configuration object', function () {
  var result = void 0;

  beforeEach(function () {
    result = (0, _index2.default)([]);
  });

  it('should contain storage settings', function () {
    (0, _expect2.default)(result.storage).toEqual([]);
  });

  it('should contain an extend object', function () {
    (0, _expect2.default)(result.extend).toEqual({
      root: {},
      context: {}
    });
  });

  it('should contain network settings', function () {
    (0, _expect2.default)(result.network).toEqual({
      clients: [],
      servers: []
    });
  });

  it('should contain a hooks object', function () {
    var hook = {
      read: [],
      write: [],
      request: [],
      update: []
    };
    (0, _expect2.default)(result.hooks).toEqual({
      before: hook,
      after: hook,
      catch: hook
    });
  });

  describe('merge', function () {

    it('should not add undefined items', function () {
      var _config = (0, _index2.default)([{}]),
          hooks = _config.hooks,
          network = _config.network,
          storage = _config.storage;

      (0, _expect2.default)(storage.length).toBe(0);
      (0, _expect2.default)(network.clients.length).toBe(0);
      (0, _expect2.default)(network.servers.length).toBe(0);
      (0, _expect2.default)(hooks.before.read.length).toBe(0);
      (0, _expect2.default)(hooks.before.write.length).toBe(0);
      (0, _expect2.default)(hooks.before.request.length).toBe(0);
      (0, _expect2.default)(hooks.before.update.length).toBe(0);
    });

    it('should add extensions', function () {
      var noop = function noop() {};
      result = (0, _index2.default)([{
        extend: {
          root: { noop: noop },
          context: { noop: noop }
        }
      }]);

      (0, _expect2.default)(result.extend.root).toContain({ noop: noop });
      (0, _expect2.default)(result.extend.context).toContain({ noop: noop });
    });

    it('should add storage', function () {
      var storage = { name: 'Storage driver' };

      var result = (0, _index2.default)([{
        storage: [storage]
      }]);

      (0, _expect2.default)(result.storage).toEqual([storage]);
    });

    it('should add network client drivers', function () {
      var client = { name: 'Network client' };
      var network = {
        clients: [client]
      };

      var result = (0, _index2.default)([{ network: network }]);
      (0, _expect2.default)(result.network.clients).toEqual([client]);
    });

    it('should add network server drivers', function () {
      var server = { name: 'Network server' };
      var network = {
        servers: [server]
      };
      var result = (0, _index2.default)([{ network: network }]);

      (0, _expect2.default)(result.network.servers).toEqual([server]);
    });

    it('should merge before hooks', function () {
      var write = function write() {};
      var hooks = {
        before: { write: write }
      };

      var result = (0, _index2.default)([{ hooks: hooks }]);

      (0, _expect2.default)(result.hooks.before.write).toEqual([write]);
    });

    it('should add network drivers in the order they\'re defined', function () {
      var first = function first() {};
      var second = function second() {};

      var result = (0, _index2.default)([{
        network: {
          clients: [first],
          servers: [first]
        }
      }, {
        network: {
          clients: [second],
          servers: [second]
        }
      }]);

      (0, _expect2.default)(result.network.clients).toEqual([first, second]);
      (0, _expect2.default)(result.network.servers).toEqual([first, second]);
    });

    it('should add storage drivers in the order they\'re defined', function () {
      var first = function first() {};
      var second = function second() {};

      var result = (0, _index2.default)([{
        storage: [first]
      }, {
        storage: [second]
      }]);

      (0, _expect2.default)(result.storage).toEqual([first, second]);
    });

    it('should add hooks in the order they\'re defined', function () {
      var first = function first() {};
      var second = function second() {};

      var result = (0, _index2.default)([{
        hooks: {
          before: { read: first }
        }
      }, {
        hooks: {
          before: { read: second }
        }
      }]);

      (0, _expect2.default)(result.hooks.before.read).toEqual([first, second]);
    });

    it('should prefer later-defined extensions when conflicts happen', function () {
      var first = function first() {};
      var second = function second() {};

      var result = (0, _index2.default)([{
        extend: {
          root: { method: first },
          context: { method: first }
        }
      }, {
        extend: {
          root: { method: second },
          context: { method: second }
        }
      }]);

      (0, _expect2.default)(result.extend.root.method).toBe(second);
      (0, _expect2.default)(result.extend.context.method).toBe(second);
    });
  });
});