'use strict';

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _mergeConfigs = require('../merge-configs');

var _mergeConfigs2 = _interopRequireDefault(_mergeConfigs);

var _index = require('./index');

var pipeline = _interopRequireWildcard(_index);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var hooks = function hooks(_hooks) {
  return (0, _mergeConfigs2.default)([{ hooks: _hooks }]);
}; /* eslint-env mocha */


describe('The pipeline\'s default option setter', function () {

  var storage = { storage: true };
  var client = { client: true };
  var config = (0, _mergeConfigs2.default)([{
    storage: [storage],
    network: {
      clients: [client]
    }
  }]);

  it('should add storage and network fields', function () {
    var options = pipeline.defaults(config, {});

    (0, _expect2.default)(options).toContain({
      storage: [storage],
      clients: [client]
    });
  });

  it('should use given storage options instead of the default', function () {
    var options = pipeline.defaults(config, {
      storage: [{ custom: true }]
    });

    (0, _expect2.default)(options).toContain({
      storage: [{ custom: true }],
      clients: [client]
    });
  });

  it('should use given network options instead of the default', function () {
    var options = pipeline.defaults(config, {
      clients: [{ custom: true }]
    });

    (0, _expect2.default)(options).toContain({
      clients: [{ custom: true }],
      storage: [storage]
    });
  });
});

describe('The pipeline\'s output formatter', function () {
  var spy = (0, _expect.createSpy)();
  var transform = pipeline.format(spy);

  beforeEach(spy.reset.bind(spy));

  it('should return new arguments when given', function () {
    var result = transform(['new arg list'], ['original list']);
    (0, _expect2.default)(result).toEqual(['new arg list']);
  });

  it('should patch missing args with the previous args', function () {
    var result = transform([undefined, 'second new item'], ['first old item', 'second old item']);
    (0, _expect2.default)(result).toEqual(['first old item', 'second new item']);
  });

  it('should call the handler when unknown args are given', function () {
    spy.andReturn(['successful']);
    var result = transform(new _set2.default(), ['old args']);

    (0, _expect2.default)(result).toEqual(['successful']);
  });
});

describe('The before.read pipeline', function () {

  it('should allow hooks to change the args', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
    var overridden, read, config, _ref3, _ref4, key;

    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            overridden = 'Haha changed your key';

            read = function () {
              var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
                return _regenerator2.default.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        return _context.abrupt('return', [overridden]);

                      case 1:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, undefined);
              }));

              return function read() {
                return _ref2.apply(this, arguments);
              };
            }();

            config = hooks({
              before: { read: read }
            });
            _context2.next = 5;
            return pipeline.before.read(config, ['original boring key']);

          case 5:
            _ref3 = _context2.sent;
            _ref4 = (0, _slicedToArray3.default)(_ref3, 1);
            key = _ref4[0];


            (0, _expect2.default)(key).toBe(overridden);

          case 9:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  describe('hook that returns a string', function () {

    it('should overwrite the key', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
      var read, config, _ref7, _ref8, key;

      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              read = function () {
                var _ref6 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
                  return _regenerator2.default.wrap(function _callee3$(_context3) {
                    while (1) {
                      switch (_context3.prev = _context3.next) {
                        case 0:
                          return _context3.abrupt('return', 'String return');

                        case 1:
                        case 'end':
                          return _context3.stop();
                      }
                    }
                  }, _callee3, undefined);
                }));

                return function read() {
                  return _ref6.apply(this, arguments);
                };
              }();

              config = hooks({
                before: { read: read }
              });
              _context4.next = 4;
              return pipeline.before.read(config, ['Original key']);

            case 4:
              _ref7 = _context4.sent;
              _ref8 = (0, _slicedToArray3.default)(_ref7, 1);
              key = _ref8[0];


              (0, _expect2.default)(key).toBe('String return');

            case 8:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, undefined);
    })));
  });

  describe('hook that returns an object', function () {

    it('should overwrite the options', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6() {
      var read, config, _ref11, _ref12, options;

      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              read = function () {
                var _ref10 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5() {
                  return _regenerator2.default.wrap(function _callee5$(_context5) {
                    while (1) {
                      switch (_context5.prev = _context5.next) {
                        case 0:
                          return _context5.abrupt('return', { overridden: true });

                        case 1:
                        case 'end':
                          return _context5.stop();
                      }
                    }
                  }, _callee5, undefined);
                }));

                return function read() {
                  return _ref10.apply(this, arguments);
                };
              }();

              config = hooks({
                before: { read: read }
              });
              _context6.next = 4;
              return pipeline.before.read(config, ['sup', {}]);

            case 4:
              _ref11 = _context6.sent;
              _ref12 = (0, _slicedToArray3.default)(_ref11, 2);
              options = _ref12[1];


              (0, _expect2.default)(options).toEqual({ overridden: true });

            case 8:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, undefined);
    })));
  });
});

describe('The pipeline', function () {

  var methods = [['before', 'read'], ['before', 'write'], ['before', 'request'], ['before', 'update'], ['after', 'read'], ['after', 'write'], ['after', 'request'], ['after', 'update']];

  methods.forEach(function (_ref13) {
    var _ref14 = (0, _slicedToArray3.default)(_ref13, 2),
        tense = _ref14[0],
        method = _ref14[1];

    describe('"' + tense + '.' + method + '" method', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee9() {
      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:

              it('should allow hooks to override the arguments', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7() {
                var args, config, result;
                return _regenerator2.default.wrap(function _callee7$(_context7) {
                  while (1) {
                    switch (_context7.prev = _context7.next) {
                      case 0:
                        args = ['original string', { options: true }];
                        config = hooks((0, _defineProperty3.default)({}, tense, (0, _defineProperty3.default)({}, method, function () {
                          return [undefined, { replaced: true }];
                        })));
                        _context7.next = 4;
                        return pipeline[tense][method](config, args);

                      case 4:
                        result = _context7.sent;

                        (0, _expect2.default)(result).toEqual(['original string', { replaced: true }]);

                      case 6:
                      case 'end':
                        return _context7.stop();
                    }
                  }
                }, _callee7, undefined);
              })));

              it('should add default options', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee8() {
                var args, config, result;
                return _regenerator2.default.wrap(function _callee8$(_context8) {
                  while (1) {
                    switch (_context8.prev = _context8.next) {
                      case 0:
                        args = ['string', { setting: true }];
                        config = hooks();
                        _context8.next = 4;
                        return pipeline[tense][method](config, args);

                      case 4:
                        result = _context8.sent;


                        (0, _expect2.default)(result).toEqual(['string', {
                          setting: true,
                          storage: [],
                          clients: []
                        }]);

                      case 6:
                      case 'end':
                        return _context8.stop();
                    }
                  }
                }, _callee8, undefined);
              })));

            case 2:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, undefined);
    })));
  });
});